/*
    program correlation_dimension

    Computes the correlation dimension via the algorithm by Grassberger-Procaccia
    - P. Grassberger & I. Procaccia. Characterization of strange attractors.
      Physical Review Letters 50, pp 346-349 (1983).
    The raw data in one column form serve as input, the output is in two columns.
    The first column stands for the logarithm of the radius of the hypersphere,
    the second column is the logarithm of the correlation integral (or the number
    of points within the hypersphere). The logarithms are due to basis 2 and can be
    changed by editing LOG_BASE in the preprocessor header. To avoid temporal
    correlation, distances between vectors were computed only when their *TEMPORAL*
    distance is greater than tau*TEMP_CORR.
    No refinements have been made in order to improve the results or the algorithm
    itself (see the literature for examples).

    The best way to start is to run the program with low number of points
    in (max-min). Estimate the largest radius (s.th. like the attractor extent
    or the time series amplitude range) and then the smallest radius. With
    decreasing radius r, the statistic will become poor! Many authors rack their
    brains over a proper choice of the time lag tau. The easiest way is to look
    for the first zero of the autocorrelation function. *Beware of the Lorenz
    attractor*, in this case you better use the principle of 'mutual information'.
    Finally you will run a series of dimension estimates by varying the embedding
    dimension, say from 2 to 20 in steps of 1.

    Good luck!  Claus
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#define  TEMP_CORR 2        /* use TEMP_CORR to avoid temporal correlation */
#define  LOG_BASE 2.0       /* basis of logarithms in output file */


void begin_message(void)
{
    printf("\n\nCorrelation dimension d2,  (Grassberger-Procaccia)\n");
    printf("by Claus D. Wagner, Department of Physiology, Charit, Berlin\n");
    printf("V0.89\t13.6.1995\n");
}

void error_message(void)
{
    printf("\nUsage\n");
    printf("corr 'infile' 'outfile' npt dim tau min max pts\n\n");
    printf("'infile': input data file in one column form.\n");
    printf("'outfile': output file in two columns:\n");
    printf("\tx-axis:\tdistance r,\n");
    printf("\ty-axis:\tnumber of points within a sphere of radius r,\n");
    printf("\t  (the logarithms are due to basis 2).\n");
    printf("npt: number of points taken from the input file.\n");
    printf("dim: embedding dimension.\n");
    printf("tau: time delay of embedding vector components (in number of points).\n");
    printf("min: minimal radius r in output file (in units of signal).\n");
    printf("max: maximal radius r in output file (in units of signal).\n");
    printf("pts: number of points in (max - min).\n\n");
    printf("type Ctrl-break to stop\n\n\n");
}

void infile_error(void)
{
    printf("\nNot enough points in input file!\n");
    exit(1);
}
main(int argc, char *argv[]) {

FILE *f1, *f2;
double absdistance, starter, stopper;
long int dim, stepper, tau;
double co, co1, dummy;
long int i, j, k, l;
long int npt, effective=0;
double *log_r, *log_theta_r, *r;
double *theta_r;
double *distance;
double *x;

if (argc != 9) {
    error_message();
    exit(1);
}
begin_message();
/* --------------------------------------------------------------------- */
npt = atol(argv[3]);
dim = atol(argv[4]);
tau = atol(argv[5]);
starter = atof(argv[6]);
stopper = atof(argv[7]);
stepper = atol(argv[8]);
/* ---------------------------------------------------------------------- */
if (( f1 = fopen(argv[1],"r")) == NULL) {
    puts("\ncannot open file\n");
    exit(1);
}
/* ---------------------------------------------------------------------- */
if ( (x = (double *)malloc(npt*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
/* ---------------------------------------------------------------------- */
if ( (log_r = (double *)malloc((stepper+1)*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
/* ---------------------------------------------------------------------- */
if ( (log_theta_r = (double *)malloc((stepper+1)*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
/* ---------------------------------------------------------------------- */
if ( (r = (double *)malloc((stepper+1)*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
/* --------------------------------------------------------------------- */
if ( (theta_r = (double *)malloc((stepper+1)*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
/* --------------------------------------------------------------------- */
if ( (distance = (double *)malloc(dim*sizeof(double))) == NULL) {
    printf("not enough memory!\n");
    exit(1);
}
f2 = fopen(argv[2],"w");
/* ------------- read in time series ----------------------------------- */
for (i = 0; i < npt; i++) {
    if ( ( fscanf(f1, "%le", &x[i]) ) == EOF) infile_error();
}
fclose(f1);
/*------------- calculate useful size of datafile ---------------------- */
npt = npt - dim * tau;
/* ------------   set the actual value of radius (r) */
co = (log(stopper)-log(starter)) / ((double)stepper*log(LOG_BASE));
for (j = 0; j <= stepper; j++) {
    dummy = log(starter)/log(LOG_BASE) + co*(double)j;
    r[j] = pow(LOG_BASE, dummy);
}

/* ------------ coint(2,i) is the correlation integral, now set zero --- */
for (j = 0; j <= stepper; j++) theta_r[j] = 0.0;
/* ------------ the very beginning ------------------------------------- */
for (i = 0; i < npt; i++) {
    /*printf("%d\n",npt-i);*/
    for (k = i+1+TEMP_CORR*tau; k < npt; k++) {
/* ------------ distance vector ---------------------------------------- */
	effective++;
        for (l = 0; l < dim; l++) {
            distance[l] = x[i+l*tau] - x[k+l*tau];
        }
/* ------------ calculating norm of distance vector -------------------- */
        absdistance = 0.0;
        for (l = 0; l < dim; l++) {
            absdistance += distance[l] * distance[l];
        }
        absdistance = sqrt(absdistance);
/* ----------- theta function   ---------------------------------------- */
        for (j = stepper; j >= 0; j--) {
            if ( absdistance <= r[j] ) theta_r[j]++;
        }
    }
}
/* --------------------------------------------------------------------- */
for (j = 0; j <= stepper; j++) {
    theta_r[j] /= (double) effective;
}
/* --------- logarithms ------------------------------------------------ */
for (j = 0; j <= stepper; j++) {
    if ( r[j] > 0.0 && theta_r[j] != 0.0 ) {
        log_theta_r[j] = log(theta_r[j]) / log(LOG_BASE);
        log_r[j] = log(r[j]) / log(LOG_BASE);
    }
}
free(x);
/* -------- output of file xxx.dim ------------------------------------- */
for (j = 0; j <= stepper; j++) {
    if ( r[j] > 0.0 && theta_r[j] != 0.0 ) {
        printf("%f %f\n", log_r[j], log_theta_r[j]);
        fprintf(f2,"%f %f\n", log_r[j], log_theta_r[j]);
    }
}
free(r);
free(theta_r);
free(log_r);
free(log_theta_r);
fclose(f2);
}   /* end of main */
